package org.fuseri.moduleexercise.question;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.fuseri.model.dto.exercise.AnswerDto;
import org.fuseri.model.dto.exercise.AnswerInQuestionCreateDto;
import org.fuseri.model.dto.exercise.ExerciseCreateDto;
import org.fuseri.model.dto.exercise.ExerciseDto;
import org.fuseri.model.dto.exercise.QuestionCreateDto;
import org.fuseri.model.dto.exercise.QuestionDto;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.util.List;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;


@SpringBootTest
@AutoConfigureMockMvc
public class QuestionTest {

    @Autowired
    ObjectMapper objectMapper;
    @Autowired
    private MockMvc mockMvc;

    public static String asJsonString(final Object obj) {
        try {
            return new ObjectMapper().writeValueAsString(obj);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Test
    void testCreateQuestion() throws Exception {
        long id = createExercise();
        var answr = new AnswerDto("dis a logical paradox", true);
        QuestionDto res = createQuestion(id);
        var expected = new QuestionDto();
        expected.setAnswers(List.of(answr));
        expected.setExerciseId(id);
        expected.setId(res.getId());
        expected.setText("this statement is false");

//        assert expected.equals(res);
    }

    private QuestionDto createQuestion(long id) throws Exception {
        var question = new QuestionCreateDto("this statement is false", id, List.of(new AnswerInQuestionCreateDto("dis a logical paradox", true)));


        var posted = mockMvc.perform(post("/questions").content(asJsonString(question)).contentType(MediaType.APPLICATION_JSON));


        var cont = posted.andReturn().getResponse().getContentAsString();
        var res = objectMapper.readValue(cont, QuestionDto.class);
        return res;
    }

    private long createExercise() {
        var postExercise = new ExerciseCreateDto("idioms", "exercise on basic idioms", 2, 0L);

        long id = 0L;

        try {
            var dis = mockMvc.perform(post("/exercises").content(asJsonString(postExercise)).contentType(MediaType.APPLICATION_JSON));
            var ok = dis.andReturn().getResponse().getContentAsString();

            var ll = objectMapper.readValue(ok, ExerciseDto.class);

            id = ll.getId();
        } catch (Exception e) {
            assert (false);
        }
        return id;
    }


    @Test
    void getQuestion() throws Exception {


        long exerciseId = createExercise();
        var question = createQuestion(exerciseId);

        var theId = String.format("/questions/%s", question.getId());


        var gets = mockMvc.perform(get(theId));

        var content = gets.andReturn().getResponse().getContentAsString();
        var res = objectMapper.readValue(content, QuestionDto.class);

        assert res.equals(question);

    }

    @Test
    void getAnswer() throws Exception {
        var exerciseId = createExercise();
        var question = createQuestion(exerciseId);

        var gets = mockMvc.perform(get(String.format("/questions/%s/answers", question.getId())));

        var content2 = gets.andReturn().getResponse().getContentAsString();

        var res = objectMapper.readValue(content2, new TypeReference<List<AnswerDto>>() {
        });

        assert (res.equals(question.getAnswers()));

    }

    @Test
    void TestUpdate() throws Exception {
        long id = createExercise();
        var question = createQuestion(id);

        var updated = """
                {
                  "text": "wat a paradox?",
                  "exerciseId": "%s"
                }
                """;

        updated = String.format(updated, id);
//
//        var smth = mockMvc.perform(put(String.format("/questions/%s", question.getId())).content(updated).contentType(MediaType.APPLICATION_JSON));
//
//        var content = smth.andReturn().getResponse().getContentAsString();

//        var res = objectMapper.readValue(content, QuestionDto.class);

//        question.setText("wat a paradox?");

//        assert (question.equals(res));

    }
}
